/**********************************************************************************
* This program controls the functions of relays on a PCBs to set volume resistors, 
* mute the output, and change the balance of the channels. It also does display 
* dimming. The controller can also inactivate the system through a control pin.
* The 8 volume relays give 256 steps of output attenuation. The mute relay is used
* to disconnect the input and ground the input to the gain section to remove any
* noise from the output. Control is either through an IR remote control or rotary 
* encoder on the front panel. The button part of the rotary encoder will activate
* the system when inactive or, when active, turn on or off muting.
* The controller is a Arduino Mega2560 and the pins numbers are relate to the board
* named pins not the pin numbers of the processor.
* Uses an outside library for the IR functions, in this case, an Apple remote
***********************************************************************************/

 #define DECODE_NEC     // Includes Apple and Onkyo
 #define IR_RECEIVE_PIN 20 // To be compatible with Mega2540 interrupts, pin 20 is chosen here.
 #include "IRremote2.hpp"  // A stripped down version of IRremote.hpp, NEC receive only.

 // Global PIN assignment constants
  // Volume Relay control output pin variables "pin 1" is LSB "pin 8" is MSB
  // Version 2 Main Analog board pin constants, pins flopped for straight connector wiring,lower board
  const int LvolRelay1=38, LvolRelay2=40, LvolRelay3=42, LvolRelay4=44,  LvolRelay5=46, LvolRelay6=48, LvolRelay7=50, LvolRelay8=52;
  // Version 1 Main Analog board pin constants, lower board
  //const int LvolRelay1=52, LvolRelay2=50, LvolRelay3=48, LvolRelay4=46,  LvolRelay5=44, LvolRelay6=42, LvolRelay7=40, LvolRelay8=38;
  // Version 1+2 Main Analog board pin constants, top board
  const int RvolRelay1=36, RvolRelay2=34, RvolRelay3=32, RvolRelay4=30,  RvolRelay5=28, RvolRelay6=26, RvolRelay7=24, RvolRelay8=22;

  // Balance LED output pin constants
  const int BalanceLEDL3=9, BalanceLEDL2=8, BalanceLEDL1=7, BalanceLEDLR0=6, BalanceLEDR1=5, BalanceLEDR2=3, BalanceLEDR3=2, StandbyLED=1;

  // Mute control and standby: Relay pin output, and Mute LED Constants
  const int muteRelayPin=69, muteLED=4, StandbyRelayPin=68; 
 
  // constants for Seven segment LED pins and Common Cathode pins with PWM 
  const int segmentA=54, segmentB=55, segmentC=56, segmentD=57, segmentE=58, segmentF=59, segmentG=60;
  const int Digit1CC=10, Digit2CC=11, Digit3CC=12;

  // Volume Encoder D62/A8/PK0/PCINT16 and D63/A9/PK1/PCINT17, Mute Button input
  const int EncoderAPin=A8, EncoderBPin=A9, muteButton=14;

  const int numeral[10] = { //dp,GFEDCBA segments to light for number
   B00111111, //0
   B00000110, //1
   B01011011, //2
   B01001111, //3
   B01100110, //4
   B01101101, //5
   B01111100, //6
   B00000111, //7
   B01111111, //8
   B01100111, //9
   };

  const int numberofDigits = 3; //number of seven segment LED display parts
  const int digitPins[numberofDigits] = {Digit1CC, Digit2CC, Digit3CC}; //digits 1, 2, 3

/* Variable assignments   */ 
  // Right and Left Volume, two volume variables for balance, 0-255, 
  // Balance: tracks where the balance is set, 1-7, 1:+3db left, 5:+2db left, 9:+1db left, 13:centered
  // 17:+1db right, 21:+2db right, 25:+3db right 
  // Mute is the relay setting 0 is off/1 is on, Dimming Display: 0-4 dimmest to brightest, 
  // Dimming LEDs: 0-100 dimmest to brightest, dimBright 
  int Rvol=0, Lvol=0, DisplayVol=0, Balance=13, mute=1, dimBright=4, brightness, Active=1;
  boolean waking=false, dimDown=true, MuteButtonActive=false;

  // variables for interrupts, have to be volatile for use in the Interrupt Service Routine and in the Main Loop
  volatile byte seqA = 0;
  volatile byte seqB = 0;
  volatile boolean volUpInt = false;
  volatile boolean volDownInt = false; 
// end of declarations
  
/***************************************************************************
* the setup function runs once when you press reset or power up the board  *
***************************************************************************/
 void setup() {
  //set volume, input and mute pins to outputs to control relays, balance LEDs, and display pins
  pinMode(LvolRelay1, OUTPUT);  //pin D38 (see variable names above)
  pinMode(LvolRelay2, OUTPUT);  //pin D40
  pinMode(LvolRelay3, OUTPUT);  //pin D42
  pinMode(LvolRelay4, OUTPUT);  //pin D44
  pinMode(LvolRelay5, OUTPUT);  //pin D46
  pinMode(LvolRelay6, OUTPUT);  //pin D48
  pinMode(LvolRelay7, OUTPUT);  //pin D50
  pinMode(LvolRelay8, OUTPUT);  //pin D52
  pinMode(RvolRelay1, OUTPUT);  //pin D22
  pinMode(RvolRelay2, OUTPUT);  //pin D24
  pinMode(RvolRelay3, OUTPUT);  //pin D26
  pinMode(RvolRelay4, OUTPUT);  //pin D28
  pinMode(RvolRelay5, OUTPUT);  //pin D30
  pinMode(RvolRelay6, OUTPUT);  //pin D32
  pinMode(RvolRelay7, OUTPUT);  //pin D34
  pinMode(RvolRelay8, OUTPUT);  //pin D36
  pinMode(muteRelayPin, OUTPUT);//pin D69
  pinMode(muteLED, OUTPUT);     //pin D4
  pinMode(BalanceLEDL3, OUTPUT); //pin D9
  pinMode(BalanceLEDL2, OUTPUT); //pin D8
  pinMode(BalanceLEDL1, OUTPUT); //pin D7
  pinMode(BalanceLEDLR0, OUTPUT); //pin D6
  pinMode(BalanceLEDR1, OUTPUT); //pin D5
  pinMode(BalanceLEDR2, OUTPUT); //pin D3
  pinMode(BalanceLEDR3, OUTPUT); //pin D2
  pinMode(segmentA, OUTPUT); //pin D54
  pinMode(segmentB, OUTPUT); //pin D55
  pinMode(segmentC, OUTPUT); //pin D56
  pinMode(segmentD, OUTPUT); //pin D57
  pinMode(segmentE, OUTPUT); //pin D58
  pinMode(segmentF, OUTPUT); //pin D59
  pinMode(segmentG, OUTPUT); //pin D60
  pinMode(Digit1CC, OUTPUT); //pin D10
  pinMode(Digit2CC, OUTPUT); //pin D11
  pinMode(Digit3CC, OUTPUT); //pin D12
  pinMode(StandbyLED, OUTPUT); //pin D1
  pinMode(StandbyRelayPin, OUTPUT); //pin D68

  //set volume Encoder and mute button pins to input
  pinMode(EncoderAPin, INPUT);    //pin A8
  pinMode(EncoderBPin, INPUT);    //pin A9
  pinMode(muteButton, INPUT);     //pin D14

  //set output pins for volume off and mute to on, standby to off
  digitalWrite(LvolRelay1, LOW);  //turn all volume relays off for vol 0
  digitalWrite(LvolRelay2, LOW);
  digitalWrite(LvolRelay3, LOW);
  digitalWrite(LvolRelay4, LOW);
  digitalWrite(LvolRelay5, LOW);
  digitalWrite(LvolRelay6, LOW);
  digitalWrite(LvolRelay7, LOW);
  digitalWrite(LvolRelay8, LOW);
  digitalWrite(RvolRelay1, LOW);
  digitalWrite(RvolRelay2, LOW);
  digitalWrite(RvolRelay3, LOW);
  digitalWrite(RvolRelay4, LOW);
  digitalWrite(RvolRelay5, LOW);
  digitalWrite(RvolRelay6, LOW);
  digitalWrite(RvolRelay7, LOW);
  digitalWrite(RvolRelay8, LOW);
  digitalWrite(muteRelayPin, HIGH); // turn mute relay on
  digitalWrite(StandbyRelayPin, HIGH); // turn standby relay on
  digitalWrite(segmentA, HIGH); //Turn segments on for a zero
  digitalWrite(segmentB, HIGH);
  digitalWrite(segmentC, HIGH);
  digitalWrite(segmentD, HIGH);
  digitalWrite(segmentE, HIGH);
  digitalWrite(segmentF, HIGH);  
  digitalWrite(segmentG, LOW);
  digitalWrite(StandbyLED, LOW);     // turn off Standby LED
    
  analogWrite(BalanceLEDL3, 0); //All balance LED turned off 
  analogWrite(BalanceLEDL2, 0);
  analogWrite(BalanceLEDL1, 0);
  analogWrite(BalanceLEDLR0, 0);
  analogWrite(BalanceLEDR1, 0);
  analogWrite(BalanceLEDR2, 0);
  analogWrite(BalanceLEDR3, 0);
  analogWrite(muteLED, 70);    //Mute LED is on
  analogWrite(Digit1CC, 255);   //Left digit Off, common cathode is active low
  analogWrite(Digit2CC, 255);   // turn middle digit off
  analogWrite(Digit3CC, 1);     // turn right digit on, set to one
 
  // setup Pin Change Interrupt enables for rotary encoder
  PCICR = 0b00000100; //PCIE2: pin Change interrupt Enable 2 PCINT23:16
  PCMSK2 = 0b00000011; //Enable PIN Change Interrupt for A8 and A9

  // Start the IR receiver and if not 3 parameters specified, take LED_BUILTIN pin from the internal boards definition as default feedback LED
  IrReceiver.begin(IR_RECEIVE_PIN, ENABLE_LED_FEEDBACK);
  //Serial.begin(115200);
  //Serial.println("End of setup");
 } //End of Setup

 /*********************************************************
 * Interrupt Service Routine for PinChange Interrupt 2    *
 * Every detent in the rotary encoder generates 4 pin     *
 * changes, 2 on Pin A and 2 on Pin B                     *
 *********************************************************/
 ISR (PCINT2_vect){ 
   //When Interrupt triggered read encoder pins
   boolean A_val = digitalRead(EncoderAPin);
   boolean B_val = digitalRead(EncoderBPin);

   //Record Pin A and B sequences in the global variables
   seqA <<= 1;     //shift the current byte date over one bit to the left
   seqA |= A_val;  //OR in, add, the new bit value, 0 or 1, of the encoder pin A
   seqB <<= 1;     //shift the current byte date over one bit to the left
   seqB |= B_val;  //OR in, add, the new bit value, 0 or 1, of the encoder pin A
   //Mask the Most Significant four bits
   seqA &= 0b00001111; // wipes off the previous four reads that were shifted 
   seqB &= 0b00001111; // to the four higher bits
   //Compare the recorded sequence with the expected sequence
   if (seqA == 0b00000011 && seqB == 0b00001001){
     volUpInt = true;
   }
   if (seqA == 0b00001001 && seqB == 0b00000011){
     volDownInt = true;
   }
 } // end ISR

 /* ***********************************
 * void loop()                       *
 *  runs over and over again forever *
 *  No input parameters              *
 *  Returns no values                * 
 ************************************/
 void loop() {
  //Booleans for task status, to be set true when a task is selected
  bool volUp=false, volDown=false, muteIn=false, balLeft=false, balRight=false, dimChange=false;
  byte command=0;

    /*
     * Check if received data is available and if yes, try to decode it.
     * Decoded result is in the IrReceiver.decodedIRData structure.
     */
    if (IrReceiver.decode()){
      // check the received data, perform actions for the received command
      command = IrReceiver.decodedIRData.command;
      command >>= 1;
      //Serial.print("command ");
      //Serial.println(command);
      //Serial.println(command,HEX);
      if(command==0x05){
        volUp=true;     //0x5=decimal 5
      }else if(command==0x06){
        volDown=true;   //0x6=decimal 6
      }else if(command==0x2E){
        dimChange=true; //0x2E=decimal 46
      }else if(command==0x2F){
        muteIn=true;    //0x2F=decimal 47
      }else if(command==0x3){
        balRight=true;  //0x3=decimal 3
      }else if(command==0x4){
        balLeft=true;   //0x4=decimal 4
      }else if(command==0x1){
        if (Active==0){
          Active=1;
        }
        else{
          Active=0; 
        }    
      }
    } // end of IR receiver/decode
 
    // check encoder button for input 
    if(MuteButtonActive == false){
      if(digitalRead(muteButton) == HIGH){
        MuteButtonActive = true;
        if (Active==0){  //when inactive Encoder button will activate
          Active=1;
          delay(200);
        }
      }
      //end of if is near end of loop

  switch (Active){  //active off=0, on=1
    case 0: if(!waking){  //going to sleep mode
              inactive();  //turn off display
              waking=true;  //set waking so next time toggle menu butto on IR will wake up
              delay(20);
            }
            break;
    case 1: if(waking){  //if Active and waking up
              activate(); //activate display
              waking=false;  //now awake, so set waking off
              delay(20);
            } else {  //already wake
              // Check the rotary encoder for movement
              if (volUpInt){  //if volume up interrupt then
                volUp = true;  //set volume up true
                volUpInt = false;  //reset volume up interrupt
              }
              if (volDownInt){  //if volume down interrupt then
                volDown = true;  //set volume down true
                volDownInt = false;  //reset volume up interrupt
              }
              // check encoder button for input 
              if(digitalRead(muteButton) == HIGH){
                muteIn=true;  //turn on mute task variable
                delay(200);  //delay for debunce for second button press
              }

              //Setup to perform whichever task is needed.
              //Screen the value to be sent to the function so they are in range
              if(volUp){
                if((Lvol < 255)&&(Rvol < 255)){  //can't go beyond 255
                   Lvol=Lvol+1;    // not at 255 yet so add 1
                   Rvol=Rvol+1;
                   vol_up(Lvol, Rvol); //Call volume up procedure
                }
                if(mute){  //if mute then unmute
                  digitalWrite(muteRelayPin, LOW); //then turn off mute relay
                  analogWrite(muteLED, 0);    //then turn off mute LED
                  mute = 0;  //reset mute volume
                  display_balance2();  //change balance LEDs
                }   
              }//end of if(volUp)

              if(volDown){
                if((Lvol>0)&&(Rvol>0)){   // Can't go less than 0
                  Lvol=Lvol-1; // is greater than 0 so subtract one
                  Rvol=Rvol-1;
                  if((Lvol==0)||(Rvol==0)){  //reached zero, reset Vol+Balance, go mute
                    Lvol=0;    // set volume to zero both sides
                    Rvol=0;
                    Balance=13;  //set balance to center
                    digitalWrite(muteRelayPin, HIGH); //mute relay on
                    mute = 1;   //set mute var on
                    analogWrite(muteLED, 128); // turn on mute LED
                    analogWrite(BalanceLEDL3, 0); //turn off all balance LEDs
                    analogWrite(BalanceLEDL2, 0);
                    analogWrite(BalanceLEDL1, 0);
                    analogWrite(BalanceLEDLR0, 0);
                    analogWrite(BalanceLEDR1, 0);
                    analogWrite(BalanceLEDR2, 0);
                    analogWrite(BalanceLEDR3, 0);
                  }
                  else{
                    if(mute){  //unmute if volume change but not zero
                      digitalWrite(muteRelayPin, LOW);
                      analogWrite(muteLED, 0);
                      mute = 0;
                      display_balance2();
                    }  
                  }
                  vol_down(Lvol, Rvol);                    
                }  
              } //end of if(volDown)

              if(muteIn){
                if (mute==0) // if mute is off then turn on
                  mute=1; 
                else
                 mute=0;  // else is on so turn off
                toggle_mute();  // call mute toggle procedure
                delay(16);
              }
  
              if(balRight){
                if(Balance<25){   // balance max is 25, and less than 25
                  if(Balance>=13){ //balance is centered, or to the right
                    if(Rvol < 255){ //turn up volume is not at max
                      Rvol=Rvol+1;  // and volume up
                      vol_up(Lvol, Rvol);  //call volume up procedure
                    }       
                  }
                  else{  //balance is to the left and moving right
                    if(Lvol>0){ //if left vol above zero, turn down left
                      Lvol=Lvol-1;
                      vol_down(Lvol, Rvol);  //call volume down procedure
                    }
                  }
                  Balance=Balance+1; //not to max so up one for balance LEDs
                  if(mute){  //unmute if vol change
                     digitalWrite(muteRelayPin, LOW);
                     analogWrite(muteLED, 0);
                     mute = 0;
                  } //end if(mute)
                  display_balance2(); //change balance LEDs
                } //end if(Balance<25)
              }//end of if(balRight)
  
              if(balLeft){
                if(Balance>1){   // balance min is 1
                  if(Balance<=13){ //balance is centered or to left
                    if(Lvol<255){  //left vol is is not max
                      Lvol=Lvol+1;   // and volume up
                      vol_up(Lvol, Rvol);  //call volume up procedure
                    }  
                  }
                  else{  //balance is to right
                    if(Rvol>0){ //is to right moving left, above 0 can turn down
                      Rvol=Rvol-1;
                      vol_down(Lvol, Rvol);  //call volume down procedure
                    }  
                  }  
                  Balance=Balance-1; //not to min so down one for balance LEDs
                  if(mute){  //unmute for balance or vol change
                    digitalWrite(muteRelayPin, LOW); //turn mute relay off
                    analogWrite(muteLED, 0);  //turn mute LED off
                    mute = 0;  //set mute var off
                  } //end if(mute)
                  display_balance2();  //change balance LEDs
                }  //end if(Balance>1)
              }//end of if(balLeft)

              if (dimChange){  // limit 0-4, 4 is full bright, 0 is off for common cathode display
                if(dimDown){ //direction is down
                  dimBright=dimBright-1;  //so reduce brightness
                  if(dimBright<0){ //make sure doesn't go below 0
                    dimBright=0;  //if so set to 0
                  }
                }
                else{  //direction is up
                    dimBright=dimBright+1; //so increase brightness
                    if(dimBright>4){  //make sure doesn't go beyond 4
                      dimBright=4;  // if so set to 4
                    }  
                  }
                if (dimChange){  // limit 0-4, 4 is full bright, 0 is off for common cathode display
                  if(dimBright == 4){ //if max then set direction to down
                    dimDown = true;
                  }
                  else if (dimBright == 0){ //if brightness has reached full dim then set up direction to up
                    dimDown = false;
                  }
                  if(mute){
                    switch(dimBright){ //set brightness of mute LED
                      case 0: analogWrite(muteLED, 0);    //Mute LED is off
                              break;
                      case 1: analogWrite(muteLED, 10);    //Mute LED is on
                              break;
                      case 2: analogWrite(muteLED, 20);    //Mute LED is on
                              break;
                      case 3: analogWrite(muteLED, 40);    //Mute LED is on
                              break;
                      case 4: analogWrite(muteLED, 80);    //Mute LED is on
                              break;       
                      default: break;                     
                    } // End Switch
                  }
                  else{ //if not mute then light balance LEDs
                    display_balance2();
                  }  
                } //end dimChange()
            }  //end dimChange()
          }  //end else already awake
          break;     
    } //end of switch

  if(Balance <= 13){ //finding which channel is dominate
    DisplayVol=Lvol;  //if balance towards left then left volume will be higher, display left volume
  }  
  else{ 
    DisplayVol=Rvol;  //if balance towards right then right volume will be higher, display right volume
  }  
  
  /* !!!Important!!! Enable receiving of the next value,
   * since receiving has stopped after the end of the current received data packet.
   */
   IrReceiver.resume(); // Enable receiving of the next value
   
  switch(dimBright){ //for dimming, 255 is off, 0 is max.
    case 0: brightness = 255;
            break;
    case 1: brightness = 150;
            break;
    case 2: brightness = 100;
            break;
    case 3: brightness = 50;
            break;
    case 4: brightness = 0;
            break;       
    default: break;                     
  } // End Switch

 } //end of if(MuteButtonActive == false){ 

  if(Active){
    showNumber(DisplayVol);
  } 

  if(digitalRead(muteButton) == LOW){
    MuteButtonActive = false;
  }

} //end of loop

/*******************************************************************
* void inactive(void)                                              *
* turn on mute relay, standby LEDs, turn off display, balance LEDs *
*******************************************************************/
void inactive(void){
  digitalWrite(StandbyLED, HIGH);
  digitalWrite(muteRelayPin, HIGH);
  digitalWrite(StandbyRelayPin, LOW);
  analogWrite(Digit1CC, 255);  
  analogWrite(Digit2CC, 255);  
  analogWrite(Digit3CC, 255);
  analogWrite(BalanceLEDL3, 0);
  analogWrite(BalanceLEDL2, 0);
  analogWrite(BalanceLEDL1, 0);
  analogWrite(BalanceLEDLR0, 0);
  analogWrite(BalanceLEDR1, 0);
  analogWrite(BalanceLEDR2, 0);
  analogWrite(BalanceLEDR3, 0);
  analogWrite(muteLED, 0);
  delay(40);
} //end of inactive()

/******************************************
* void activate(void)                     *
* turn off mute relay and Standby LED,    *               
* turn on display and balance LEDs        *
******************************************/
void activate(void){
  analogWrite(StandbyLED, 0);
  digitalWrite(StandbyRelayPin, HIGH);
  if (mute){
    digitalWrite(muteRelayPin, HIGH);
    analogWrite(muteLED, 70);
  }
  else{
    digitalWrite(muteRelayPin, LOW);
    analogWrite(muteLED, 0);
    display_balance2();
  }
  showNumber(DisplayVol);
  delay(20);
} //end of activate()

/***********************************************************************
 * Function: void vol_up(int LeftVolume, int RightVolume)              *
 *                                                                     *
 * Inputs: volume, is an integer between 0 and 255                     *
 * volume should already be screened for values of 255 or less         *
 *                                                                     *
 * Returns: None                                                       *
 *                                                                     *
 * The function creates a binary array from the volume integer         *
 * The binary values of the array are used to set the relays on, HIGH, *
 * or off, LOW, from Least Significant to Most Significant             *  
 **********************************************************************/
 void vol_up(int LeftVolume, int RightVolume){
  int LeftarryLoc=0, RightarryLoc=0;
  int LeftbinaryArry[8]={0,0,0,0,0,0,0,0}, RightbinaryArry[8]={0,0,0,0,0,0,0,0};            
  
  while(LeftVolume>0){  // keep working until volume divided to zero
    LeftbinaryArry[LeftarryLoc]=LeftVolume%2;  //take the modulus of the binary division, either 0 or 1
    LeftVolume=LeftVolume/2;  //divide by 2 because we are doing binary.
    LeftarryLoc++;  // next array location
   } //End while
    
  while(RightVolume>0){
    RightbinaryArry[RightarryLoc]=RightVolume%2;
    RightVolume=RightVolume/2;
    RightarryLoc++;
   } //End while

  // Left Channel
  if(LeftbinaryArry[0]==1){
     digitalWrite(LvolRelay1, HIGH);
  }
  else{
     digitalWrite(LvolRelay1, LOW);
  }
  if(LeftbinaryArry[1]==1){
    delay(3);
    digitalWrite(LvolRelay2, HIGH);
  }
  else{
    delay(3);
    digitalWrite(LvolRelay2, LOW);
  }
  if(LeftbinaryArry[2]==1){
    delay(3);
    digitalWrite(LvolRelay3, HIGH);
  } 
  else{
    delay(3);
    digitalWrite(LvolRelay3, LOW);
  } 
  if (LeftbinaryArry[3]==1){
    delay(3);
    digitalWrite(LvolRelay4, HIGH);
  } 
  else{
    delay(3);
    digitalWrite(LvolRelay4, LOW);
  } 
  if (LeftbinaryArry[4]==1){
    delay(3);
    digitalWrite(LvolRelay5, HIGH);
  } 
  else{
    delay(3);
    digitalWrite(LvolRelay5, LOW);
  } 
  if(LeftbinaryArry[5]==1){
    delay(3);
    digitalWrite(LvolRelay6, HIGH);
  } 
  else{
    delay(3);
    digitalWrite(LvolRelay6, LOW);
  } 
  if(LeftbinaryArry[6]==1){
    delay(3);
    digitalWrite(LvolRelay7, HIGH);
  }
  else{
    delay(3);
    digitalWrite(LvolRelay7, LOW);
  }
  if (LeftbinaryArry[7]==1){ 
    delay(3);
    digitalWrite(LvolRelay8, HIGH);
  }
  else {
    delay(3);
    digitalWrite(LvolRelay8, LOW);
  }

  // Right Channel
  if(RightbinaryArry[0]==1){
     digitalWrite(RvolRelay1, HIGH);
  }
  else{
     digitalWrite(RvolRelay1, LOW);
  }
  if(RightbinaryArry[1]==1){
    delay(3);
    digitalWrite(RvolRelay2, HIGH);
  }
  else{
    delay(3);
    digitalWrite(RvolRelay2, LOW);
  }
  if(RightbinaryArry[2]==1){
    delay(3);
    digitalWrite(RvolRelay3, HIGH);
  } 
  else{
    delay(3);
    digitalWrite(RvolRelay3, LOW);
  } 
  if (RightbinaryArry[3]==1){
    delay(3);
    digitalWrite(RvolRelay4, HIGH);
  } 
  else{
    delay(3);
    digitalWrite(RvolRelay4, LOW);
  } 
  if (RightbinaryArry[4]==1){
    delay(3);
    digitalWrite(RvolRelay5, HIGH);
  } 
  else{
    delay(3);
    digitalWrite(RvolRelay5, LOW);
  } 
  if(RightbinaryArry[5]==1){
    delay(3);
    digitalWrite(RvolRelay6, HIGH);
  } 
  else{
    delay(3);
    digitalWrite(RvolRelay6, LOW);
  } 
  if(RightbinaryArry[6]==1){
    delay(3);
    digitalWrite(RvolRelay7, HIGH);
  }
  else{
    delay(3);
    digitalWrite(RvolRelay7, LOW);
  }
  if (RightbinaryArry[7]==1){ 
    delay(3);
    digitalWrite(RvolRelay8, HIGH);
  }
  else {
    delay(3);
    digitalWrite(RvolRelay8, LOW);
  }
 } //end void vol_up()

 /********************************************************************** 
 * Function: void vol_down(int LeftVolume, int RightVolume)            *
 *                                                                     *
 * Inputs: volume, is an integer between 0 and 255                     *
 * volume should already be screened for values of 0 or more           *
 *                                                                     *
 * Returns: None                                                       *
 *                                                                     *
 * The function creates a binary array from the volume integer         *
 * The binary values of the array are used to set the relays on, HIGH, *
 * or off, LOW, from Most Significant to Least Significant             *
 **********************************************************************/
 void vol_down(int LeftVolume, int RightVolume){
  int LeftarryLoc=0, RightarryLoc=0;
  int LeftbinaryArry[8]={0,0,0,0,0,0,0,0}, RightbinaryArry[8]={0,0,0,0,0,0,0,0};
  
  while(LeftVolume>0){
    LeftbinaryArry[LeftarryLoc]=LeftVolume%2;
    LeftVolume=LeftVolume/2;
    LeftarryLoc++;
   } //End while
    
  while(RightVolume>0){
    RightbinaryArry[RightarryLoc]=RightVolume%2;
    RightVolume=RightVolume/2;
    RightarryLoc++;
   } //End while

  // Left Channel
  if (LeftbinaryArry[7] == 1){
     digitalWrite(LvolRelay8, HIGH);
  }  
  else{
     digitalWrite(LvolRelay8, LOW);
  }  
  if (LeftbinaryArry[6]==1){
     delay(2);
     digitalWrite(LvolRelay7, HIGH);
  }
  else{
    delay(2);
    digitalWrite(LvolRelay7, LOW);
  }
  if (LeftbinaryArry[5]==1){
    delay(2);
    digitalWrite(LvolRelay6, HIGH);
  }
  else{
    delay(2);
    digitalWrite(LvolRelay6, LOW);
  }
  if (LeftbinaryArry[4]==1){
    delay(2);
    digitalWrite(LvolRelay5, HIGH);
  }
  else{
    delay(2);
    digitalWrite(LvolRelay5, LOW);
  }
  if (LeftbinaryArry[3]==1){
    delay(2);
    digitalWrite(LvolRelay4, HIGH);
  }
  else{
    delay(2);
    digitalWrite(LvolRelay4, LOW);
  }
  if (LeftbinaryArry[2]==1){
    delay(2);
    digitalWrite(LvolRelay3, HIGH);
  }
  else{
    delay(2);
    digitalWrite(LvolRelay3, LOW);
  }
  if (LeftbinaryArry[1]==1){
    delay(2);
    digitalWrite(LvolRelay2, HIGH);
  }
  else{
    delay(2);
    digitalWrite(LvolRelay2, LOW);
  }
  if (LeftbinaryArry[0]==1){
    delay(2);
    digitalWrite(LvolRelay1, HIGH);
  }
  else{
    delay(2);
    digitalWrite(LvolRelay1, LOW);
  }

  // Right Channel
     if (RightbinaryArry[7] == 1){
     digitalWrite(RvolRelay8, HIGH);
  }  
  else{
     digitalWrite(RvolRelay8, LOW);
  }  
  if (RightbinaryArry[6]==1){
     delay(2);
     digitalWrite(RvolRelay7, HIGH);
  }
  else{
    delay(2);
    digitalWrite(RvolRelay7, LOW);
  }
  if (RightbinaryArry[5]==1){
    delay(2);
    digitalWrite(RvolRelay6, HIGH);
  }
  else{
    delay(2);
    digitalWrite(RvolRelay6, LOW);
  }
  if (RightbinaryArry[4]==1){
    delay(2);
    digitalWrite(RvolRelay5, HIGH);
  }
  else{
    delay(2);
    digitalWrite(RvolRelay5, LOW);
  }
  if (RightbinaryArry[3]==1){
    delay(2);
    digitalWrite(RvolRelay4, HIGH);
  }
  else{
    delay(2);
    digitalWrite(RvolRelay4, LOW);
  }
  if (RightbinaryArry[2]==1){
    delay(2);
    digitalWrite(RvolRelay3, HIGH);
  }
  else{
    delay(2);
    digitalWrite(RvolRelay3, LOW);
  }
  if (RightbinaryArry[1]==1){
    delay(2);
    digitalWrite(RvolRelay2, HIGH);
  }
  else{
    delay(2);
    digitalWrite(RvolRelay2, LOW);
  }
  if (RightbinaryArry[0]==1){
    delay(2);
    digitalWrite(RvolRelay1, HIGH);
  }
  else{
    delay(2);
    digitalWrite(RvolRelay1, LOW);
  }
 } //End void vol_down(int volume)

 /************************************************************* 
 * Funtion: void toggle_mute(void)                            *
 *                                                            *
 * Inputs:  None                                              *
 * Returns: None                                              *
 *                                                            *
 * if mute is 1 then set pin to HIGH to turn on the relay     *
 * turn the mute LED on, call display_balance to turn off     *
 * the balance LEDs                                           *
 * if mute is 0 then set pin to LOW to turn off the relay     * 
 * turn the mute LED off, call display_balance to turn on     *
 * the balance LEDs                                           *
 *************************************************************/  
 void toggle_mute(void){
  
  if ((Lvol == 0)||(Rvol==0)){
    digitalWrite(muteRelayPin, HIGH);
    analogWrite(muteLED, (100-dimBright));
    mute = 1;
   } 
  else{
    if (mute){
      digitalWrite(muteRelayPin, HIGH);
      analogWrite(muteLED, (100-dimBright));
      analogWrite(BalanceLEDL3, 0);
      analogWrite(BalanceLEDL2, 0);
      analogWrite(BalanceLEDL1, 0);
      analogWrite(BalanceLEDLR0, 0);
      analogWrite(BalanceLEDR1, 0);
      analogWrite(BalanceLEDR2, 0);
      analogWrite(BalanceLEDR3, 0);
    }
    else{
    digitalWrite(muteRelayPin, LOW);
    analogWrite(muteLED, 0);
    display_balance2();
    }
  }  
  delay(30);  
} //End void toggle_mute(void)

 /******************************************************************* 
 * Function: void display_balance2(void)                            *
 *                                                                  *
 * Inputs: None                                                     *
 * Returns: None                                                    *
 *                                                                  *
 * Sets Balance LED Display, starts at step 13, Center LED full     *
 * bright, then fades center down and next one up                   *
 *******************************************************************/
 void display_balance2(void){
  int fulldim, threequarterdim, halfdim, onequarterdim, zerodim=0;
  switch(dimBright){
    case 0: fulldim = 0;
            threequarterdim = 0;
            halfdim = 0;
            onequarterdim = 0;
            break;
    case 1: fulldim = 15;
            threequarterdim = 10;
            halfdim = 5;
            onequarterdim = 2;
            break;
    case 2: fulldim = 20;
            threequarterdim = 15;
            halfdim = 10;
            onequarterdim = 5;
            break;
    case 3: fulldim = 30;
            threequarterdim = 20;
            halfdim = 15;
            onequarterdim = 10;
            break;
    case 4: fulldim = 40;
            threequarterdim = 25;
            halfdim = 20;
            onequarterdim = 15;
            break;       
  default: break;                     
  } // End Switch
  
  switch(Balance){
    case 1: analogWrite(BalanceLEDL3, fulldim);
            analogWrite(BalanceLEDL2, fulldim);
            analogWrite(BalanceLEDL1, fulldim);
            analogWrite(BalanceLEDLR0, fulldim);
            break;
    case 2: analogWrite(BalanceLEDL3, threequarterdim);
            analogWrite(BalanceLEDL2, fulldim);
            analogWrite(BalanceLEDL1, fulldim);
            analogWrite(BalanceLEDLR0, fulldim);
            break;
    case 3: analogWrite(BalanceLEDL3, halfdim);
            analogWrite(BalanceLEDL2, fulldim);
            analogWrite(BalanceLEDL1, fulldim);
            analogWrite(BalanceLEDLR0, fulldim);
            break;
    case 4: analogWrite(BalanceLEDL3, onequarterdim);
            analogWrite(BalanceLEDL2, fulldim);
            analogWrite(BalanceLEDL1, fulldim);
            analogWrite(BalanceLEDLR0, fulldim);
            break;
    case 5: analogWrite(BalanceLEDL3, zerodim);
            analogWrite(BalanceLEDL2, fulldim);
            analogWrite(BalanceLEDL1, fulldim);
            analogWrite(BalanceLEDLR0, fulldim);
            break; 
    case 6: analogWrite(BalanceLEDL3, zerodim);
            analogWrite(BalanceLEDL2, threequarterdim);
            analogWrite(BalanceLEDL1, fulldim);
            analogWrite(BalanceLEDLR0, fulldim);
            break; 
    case 7: analogWrite(BalanceLEDL3, zerodim);
            analogWrite(BalanceLEDL2, halfdim);
            analogWrite(BalanceLEDL1, fulldim);
            analogWrite(BalanceLEDLR0, fulldim);
            break;
    case 8: analogWrite(BalanceLEDL3, zerodim);
            analogWrite(BalanceLEDL2, onequarterdim);
            analogWrite(BalanceLEDL1, fulldim);
            analogWrite(BalanceLEDLR0, fulldim);
            break;
    case 9: analogWrite(BalanceLEDL3, zerodim);
            analogWrite(BalanceLEDL2, zerodim);
            analogWrite(BalanceLEDL1, fulldim);
            analogWrite(BalanceLEDLR0, fulldim);
            break;
    case 10: analogWrite(BalanceLEDL3, zerodim);
             analogWrite(BalanceLEDL2, zerodim);
             analogWrite(BalanceLEDL1, threequarterdim);
             analogWrite(BalanceLEDLR0, fulldim);
             break;
    case 11: analogWrite(BalanceLEDL3, zerodim);
             analogWrite(BalanceLEDL2, zerodim);
             analogWrite(BalanceLEDL1, halfdim);
             analogWrite(BalanceLEDLR0, fulldim);
             break; 
    case 12: analogWrite(BalanceLEDL3, zerodim);
             analogWrite(BalanceLEDL2, zerodim);
             analogWrite(BalanceLEDL1, onequarterdim);
             analogWrite(BalanceLEDLR0, fulldim);
             break;
    case 13: analogWrite(BalanceLEDL3, zerodim);
             analogWrite(BalanceLEDL2, zerodim);
             analogWrite(BalanceLEDL1, zerodim);
             analogWrite(BalanceLEDLR0, fulldim);
             analogWrite(BalanceLEDR1, zerodim);
             analogWrite(BalanceLEDR2, zerodim);
             analogWrite(BalanceLEDR3, zerodim);
             break;
    case 14: analogWrite(BalanceLEDLR0, fulldim);
             analogWrite(BalanceLEDR1, onequarterdim);
             analogWrite(BalanceLEDR2, zerodim);
             analogWrite(BalanceLEDR3, zerodim);
             break;
    case 15: analogWrite(BalanceLEDLR0, fulldim);
             analogWrite(BalanceLEDR1, halfdim);
             analogWrite(BalanceLEDR2, zerodim);
             analogWrite(BalanceLEDR3, zerodim);
             break;
    case 16: analogWrite(BalanceLEDLR0, fulldim);
             analogWrite(BalanceLEDR1, threequarterdim);
             analogWrite(BalanceLEDR2, zerodim);
             analogWrite(BalanceLEDR3, zerodim);
             break;
    case 17: analogWrite(BalanceLEDLR0, fulldim);
             analogWrite(BalanceLEDR1, fulldim);
             analogWrite(BalanceLEDR2, zerodim);
             analogWrite(BalanceLEDR3, zerodim);
             break; 
    case 18: analogWrite(BalanceLEDLR0, fulldim);
             analogWrite(BalanceLEDR1, fulldim);
             analogWrite(BalanceLEDR2, onequarterdim);
             analogWrite(BalanceLEDR3, zerodim);
             break;
    case 19: analogWrite(BalanceLEDLR0, fulldim);
             analogWrite(BalanceLEDR1, fulldim);
             analogWrite(BalanceLEDR2, halfdim);
             analogWrite(BalanceLEDR3, zerodim);
             break;
    case 20: analogWrite(BalanceLEDLR0, fulldim);
             analogWrite(BalanceLEDR1, fulldim);
             analogWrite(BalanceLEDR2, threequarterdim);
             analogWrite(BalanceLEDR3, zerodim);
             break;
    case 21: analogWrite(BalanceLEDLR0, fulldim);
             analogWrite(BalanceLEDR1, fulldim);
             analogWrite(BalanceLEDR2, fulldim);
             analogWrite(BalanceLEDR3, zerodim);
             break;
    case 22: analogWrite(BalanceLEDLR0, fulldim);
             analogWrite(BalanceLEDR1, fulldim);
             analogWrite(BalanceLEDR2, fulldim);
             analogWrite(BalanceLEDR3, onequarterdim);
             break;
    case 23: analogWrite(BalanceLEDLR0, fulldim);
             analogWrite(BalanceLEDR1, fulldim);
             analogWrite(BalanceLEDR2, fulldim);
             analogWrite(BalanceLEDR3, halfdim);
              break; 
    case 24: analogWrite(BalanceLEDLR0, fulldim);
             analogWrite(BalanceLEDR1, fulldim);
             analogWrite(BalanceLEDR2, fulldim);
             analogWrite(BalanceLEDR3, threequarterdim);
             break;
    case 25: analogWrite(BalanceLEDLR0, fulldim);
             analogWrite(BalanceLEDR1, fulldim);
             analogWrite(BalanceLEDR2, fulldim);
             analogWrite(BalanceLEDR3, fulldim);
             break;
    default: break;                     
   } // End Switch
  delay(10);
 } // end void balance_change2(void)

 void showNumber (unsigned long number){ //number should be 5 digits
  if (number == 0)  //if number is zero only process once
    showDigit(0, numberofDigits - 1); //display 0 in the rightmost digit
  else{
    for (int digit = numberofDigits - 1; digit >= 0; digit--){ //starts at right most digit
      if (number > 0){ //Don't show leading zeros
        showDigit(number % 10, digit); // Send next digit and digit position
        number = number / 10;  // Remove the digit so only ones to show left
      }//end if
    }//end for
  }//end else
}//end showNumber()

//Displays given number on a 7-segment display at the given digit position
void showDigit (int numberx, int digitx){
  analogWrite(digitPins[digitx], brightness); // turn digit on
  PORTF = numeral[numberx]; // write port F
  delay(6); //controls illumination, time lit before off
  analogWrite(digitPins[digitx], 255); // Turn digit off
}//end showDigit()


